/*
	Copyright (C) 2020-2022 Lewis ANESA.

	This file is part of ladspa-tool-kit

	ladspa-tool-kit is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	ladspa-tool-kit is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with ladspa-tool-kit.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef __LTK_INSTANCE_H__
#define __LTK_INSTANCE_H__

#include <stdarg.h>
#include <stdint.h>

/* LTKConfig_t corresponding macros */
#define LTKCFG_GET						(UINT64_C(0) << 0)
#define LTKCFG_SET						(UINT64_C(1) << 0)
#define LTKCFG_GET_ALL					~LTKCFG_SET
#define LTKCFG_FREE_LAST_POOL_IF_EMPTY	(UINT64_C(1) << 1)
#define LTKCFG_ALLOC_ON_CLASS_CREATION	(UINT64_C(1) << 2)
#define LTKCFG_SIZE_ON_CLASS_CREATION	(UINT64_C(1) << 3)
#define LTKCFG_LOG_FD_ERROR				(UINT64_C(1) << 4)
#define LTKCFG_LOG_FD_WARNING			(UINT64_C(1) << 5)
#define LTKCFG_LOG_FD_INFO				(UINT64_C(1) << 6)

/* LTKLog macros */
#define LTK_LOG_ERROR(format, ...)		(LTKLog(0, __FILE__, __LINE__, \
										__func__, format __VA_OPT__(,) \
										__VA_ARGS__))
#define LTK_LOG_WARNING(format, ...)	(LTKLog(1, __FILE__, __LINE__, \
										__func__, format __VA_OPT__(,) \
										__VA_ARGS__))
#define LTK_LOG_INFO(format, ...)		(LTKLog(2, __FILE__, __LINE__, \
										__func__, format __VA_OPT__(,) \
										__VA_ARGS__))

/* General class type definition */
typedef struct LTKClassTypeDefinition_s LTKClassTypeDefinition_t;
/* General method definition */
typedef void (*LTKFunction_t) (void *, va_list);
/* Free event callback, called before freeing instance/class memory */
typedef void (*LTKFreeHandler_t) (void *);

typedef struct LTKConfig_s {
	uint64_t _mask;
	uint64_t _def_buf_size;
	int _log_fd_error;
	int _log_fd_warning;
	int _log_fd_info;
} LTKConfig_t;

LTKClassTypeDefinition_t *LTKAddClass(LTKClassTypeDefinition_t *
	, LTKFunction_t *, uint64_t
);
int LTKSetFreeInstanceCallback(LTKClassTypeDefinition_t *, LTKFreeHandler_t);
int LTKSetFreeClassCallback(LTKClassTypeDefinition_t *
	, LTKFreeHandler_t, void *
);
LTKClassTypeDefinition_t *LTKGetClass(void *);


void *LTKAdd (LTKClassTypeDefinition_t *);
int LTKClear (void *);
int LTKFree (void *);

int LTKRun(void *, uint64_t, ...);
int LTKRunForEach(LTKClassTypeDefinition_t *, int, uint64_t, ...);
int LTKForEach(LTKClassTypeDefinition_t *, int, LTKFunction_t, ...);

int LTKvRun(void *, uint64_t, va_list);
int LTKvRunForEach(LTKClassTypeDefinition_t *, int, uint64_t, va_list);
int LTKvForEach(LTKClassTypeDefinition_t *, int, LTKFunction_t, va_list);

LTKFunction_t LTKMember(LTKClassTypeDefinition_t *, uint64_t);

uint64_t *LTKGetObjectID (void *);
void *LTKGetObjectFromID (LTKClassTypeDefinition_t *, uint64_t);

int LTKConfigure (uint64_t, LTKConfig_t *);
int LTKLog (unsigned int, char *, int, const char *, char *, ...);

#endif /* __LTK_INSTANCE_H__ */

